# src/spacebridge_mcp/resources.py
"""
MCP Resource definitions for SpaceBridge integration.

Resource handler functions are defined and registered in server.py using FastMCP decorators.
"""

# Keep necessary imports if models were defined here, otherwise can be minimal.
# Currently, no models are defined here.
